/*
Stochastic Theory Education through Visualization Environment (STEVE) 2.0
Robert Florence, Tennessee Tech University
Visualization.h - This class creates a visualization from the output of the
simulation program and draws it using OpenGL.
Last Modified: July 24, 2008
*/

#ifndef VISUALIZATION_H
#define VISUALIZATION_H

#include <windows.h>
#include <gl/glut.h>
#include <cstdlib>
#include <fstream>
#include "Color.h"

//dimensions of the map
#define MAP_WIDTH 20
#define MAP_HEIGHT 12
//GL coordinates of the top-left corner of each map
#define MAP1_LEFT -1.3f
#define MAP1_TOP 1.4f
#define MAP2_LEFT -1.3f
#define MAP2_TOP -0.2f

class Visualization
{
    public:
        /*
        create a new three-dimensional map from the
            output of the simulation program
        period: number of timesteps in the visualization
        filename: file to read simulation program output from
        */
        Visualization(int period, char* filename);
        /*
        remove the visualization from memory
        */
        ~Visualization();
        /*
        draw the map data at the current frame
        left, top: either MAP1_LEFT, MAP1_TOP or MAP2_LEFT, MAP2_TOP
        */
        void draw(float left, float top);
        /*
        draw the map title, axes, and grid
        left, top: either MAP1_LEFT, MAP1_TOP or MAP2_LEFT, MAP2_TOP
        */
        static void drawAxes(float left, float top);
        /*
        draw the color bar on the right of the window
        */
        static void drawColorBar();
        /*
        move to previous or next frame
        */
        void decFrame();
        void incFrame();
        
    private:
        //three-dimensional array containing the output of the simulation program
        float*** map;
        //number of timesteps in the visualization
        int simPeriod;
        //current timestep in the visualization
        int frame;
        /*
        determine the color of the visualization map of the specified output value
        val: value of a grid on the map
        return: color for the value according to the color bar
        */
        Color getColor(float val);
        /*
        draw text to the window
        x, y: GL coordinates to draw the text
        font: GLUT font to draw the text in
        text: string to draw
        */
        static void drawText(float x, float y, void* font, char* text);
};

#endif
